@@ -1,6 +1,43 @@
 #[doc(hidden)] #[macro_export]
-macro_rules! __ffi_export__ {(
-    $($(#[doc = $doc:expr])+)?
+macro_rules! __ffi_export__ {
+    (
+        $(#[$($meta:tt)*])*
+        $pub:vis
+        $enum_or_struct:ident
+        $name:ident
+        {
+            $($tt:tt)*
+        }
+    ) => (
+        $(#[$($meta)*])*
+        $pub
+        $enum_or_struct
+        $name
+        {
+            $($tt)*
+        }
+
+        $crate::__cfg_headers__! {
+            $crate::inventory::submit! {
+                #![crate = $crate]
+                $crate::FfiExport({
+                    #[allow(unused_parens, clippy::all)]
+                    fn typedef (
+                        definer: &'_ mut dyn $crate::headers::Definer,
+                    ) -> $crate::std::io::Result<()>
+                    {
+                        <
+                            <$name as $crate::layout::ReprC>::CLayout
+                            as
+                            $crate::layout::CType
+                        >::c_define_self(definer)
+                    }
+                    typedef
+                })
+            }
+        }
+    );
+    ($($(#[doc = $doc:expr])+)?
     // $(#[$meta:meta])*
     $pub:vis
     $(unsafe $(@$hack:ident@)?)?
@@ -31,7 +68,7 @@ macro_rules! __ffi_export__ {(
     )?
         $body
 
-    #[allow(dead_code, nonstandard_style, unused_parens)]
+    #[allow(dead_code, nonstandard_style, unused_parens, clippy::all)]
     const _: () = {
         $($(#[doc = $doc])+)?
         #[no_mangle]
@@ -116,7 +153,7 @@ macro_rules! __ffi_export__ {(
         $crate::inventory::submit! {
             #![crate = $crate]
             $crate::FfiExport({
-                #[allow(unused_parens)]
+                #[allow(unused_parens, clippy::all)]
                 fn typedef $(<$($lt $(: $sup_lt)?),*>)? (
                     definer: &'_ mut dyn $crate::headers::Definer,
                 ) -> $crate::std::io::Result<()>
@@ -194,10 +231,10 @@ macro_rules! __ffi_export__ {(
                         )?;
                         has_args |= true;
                     )*
-                    if has_args.not() {
+                    if ! has_args {
                         out.write_all(b"void")?;
                     }
-                    drop(has_args);
+                    $crate::core::mem::drop(has_args);
                     $crate::std::io::Write::write_all(out,
                         ");\n\n"
                             .as_bytes()
